#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "aiot_state_api.h"
#include "aiot_device_api.h"
#include "demo_config.h"

int main(int argc, char *argv[])
{
    int32_t     res = STATE_SUCCESS;
    aiot_bootstrap_info_t connect_info;
    uint32_t timeout_ms = 5000;
    /* 配置SDK的底层依赖 */
    aiot_sysdep_init(aiot_sysdep_get_portfile());

    /* 创建设备 */
    void *device_client = aiot_device_create(product_key, device_name);
    if (device_client == NULL) {
        printf("device_client failed\n");
        return -1;
    }
    res = aiot_device_bootstrap_request(device_client, MQTT_PROTOCOL, timeout_ms, &connect_info);
    if(res == STATE_SUCCESS){
        printf("update host %s,port %d\r\n", connect_info.host, connect_info.port);
    }
    /* 设置设备密钥 */
    aiot_device_set_device_secret(device_client, device_secret);

    /* 连接配置参数初始化 */
    aiot_linkconfig_t* config = aiot_linkconfig_init(protocol);
    /* 设置服务器的host、port */
    aiot_linkconfig_host(config, connect_info.host, connect_info.port);

    /* 设置设备连接参数 */
    aiot_device_set_linkconfig(device_client, config);
    /* 设备建连 */
    res = aiot_device_connect(device_client);
    if (res < STATE_SUCCESS) {
        /* 尝试建立连接失败, 销毁MQTT实例, 回收资源 */
        aiot_linkconfig_deinit(&config);
        aiot_device_delete(&device_client);
        printf("aiot_device_connect failed: -0x%04X\n\r\n", -res);
        return -1;
    }

    res = aiot_device_disconnect(device_client);
    if (res < STATE_SUCCESS) {
        printf("aiot_device_disconnect failed: -0x%04X\n", -res);
        return -1;
    }

    /* 销毁设备实例, 一般不会运行到这里 */
    aiot_device_delete(&device_client);
    aiot_linkconfig_deinit(&config);
    return 0;
}
