#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/time.h>
#include "aiot_state_api.h"
#include "aiot_linkconfig_api.h"
#include "aiot_device_api.h"
#include "demo_config.h"

int main(int argc, char *argv[])
{
    int32_t     res = STATE_SUCCESS;
    void *device_client = NULL;
    aiot_linkconfig_t* config = NULL;
    
    /* 配置SDK的底层依赖 */
    aiot_sysdep_init(aiot_sysdep_get_portfile());

    /*======================== 动态获取密钥(一型一密)预注册 ===================================*/
    /* 创建设备 */
    device_client = aiot_device_create(product_key, device_name);
    if (device_client == NULL) {
        printf("device_client failed\n");
        return -1;
    }

    config = aiot_linkconfig_init(protocol);
    if(config == NULL){
        aiot_device_delete(&device_client);
        printf("aiot_linkconfig_init failed\n");
        return -1;
    }
    aiot_linkconfig_host(config, host, port);
    /* 动态注册功能需要使用product_secret, 如果是企业实例设备还需要INSTANCE_ID */
    aiot_linkconfig_dynamic_register(config, product_secret, INSTANCE_ID);

    /* 动态注册获取密钥成功，可以使用该密钥进行连接 */
    char device_secret_result[128];
    res = aiot_device_dynamic_secret(device_client, config, device_secret_result, 128);
    if(res != STATE_SUCCESS) {
        aiot_device_delete(&device_client);
        aiot_linkconfig_deinit(&config);
        printf("aiot_device_dynamic_register failed\n");
        return -1;
    }

    /* TODO: 示例只做打印处理，用户应保存设备密钥 */
    printf("dynamic register success, device_secret %s\r\n", device_secret_result);
    aiot_device_delete(&device_client);
    aiot_linkconfig_deinit(&config);

    /*======================== 动态注册(一型一密)免预注册 ==================================*/
    /* 创建设备，设备名可以动态生成, 也可使用设备唯一标识码 */
    char dynreg_device_name[64];
    struct timeval time;
    memset(&time, 0, sizeof(struct timeval));
    gettimeofday(&time, NULL);
    memset(dynreg_device_name, 0, sizeof(dynreg_device_name));
    snprintf(dynreg_device_name, sizeof(dynreg_device_name) - 1, "dynreg_demo_device_%d", (int)time.tv_usec);

    device_client = aiot_device_create(product_key, dynreg_device_name);
    if (device_client == NULL) {
        printf("device_client failed\n");
        return -1;
    }

    config = aiot_linkconfig_init(protocol);
    if(config == NULL){
        aiot_device_delete(&device_client);
        printf("aiot_linkconfig_init failed\n");
        return -1;
    }
    aiot_linkconfig_host(config, host, port);
    /* 动态注册功能需要使用product_secret, 如果是企业实例设备还需要INSTANCE_ID */
    aiot_linkconfig_dynamic_register(config, product_secret, INSTANCE_ID);

    /* 免白名单动态注册设备 */
    aiot_dynamic_register_info_t info;
    res = aiot_device_dynamic_register(device_client, config, &info);
    if(res != STATE_SUCCESS) {
        aiot_device_delete(&device_client);
        aiot_linkconfig_deinit(&config);
        printf("aiot_device_dynamic_register failed\n");
        return -1;
    }
    
    /* TODO: 此处只做打印处理，用户应该完成建连信息的保存 */
    printf("dynamic register success, username %s, password %s, clientid %s\r\n", info.username, info.password, info.client_id);
    
    /* 使用获取的username, password, clientid进行连接 */
    aiot_linkconfig_mqtt_auth(config, info.username, info.password, info.client_id);
    aiot_device_set_linkconfig(device_client, config);
    
    aiot_device_connect(device_client);

    /* 完成建连后，与普通认证方式设备操作相同 */

    res = aiot_device_disconnect(device_client);
    if (res < STATE_SUCCESS) {
        /* 尝试建立连接失败, 销毁MQTT实例, 回收资源 */
        aiot_linkconfig_deinit(&config);
        aiot_device_delete(&device_client);
        printf("aiot_device_connect failed: -0x%04X\n\r\n", -res);
        return -1;
    }
    /* 资源回收 */
    aiot_linkconfig_deinit(&config);
    aiot_device_delete(&device_client);
    return 0;
}
