#include <stdio.h>
#include <unistd.h>
#include "aiot_device_api.h"
#include "aiot_devinfo_api.h"
#include "demo_config.h"

static void* demo_device_init(char *product_key, char *device_name, char *device_secret, char *host, uint16_t port)
{
    int32_t res = STATE_SUCCESS;
    /* 创建设备 */
    void *device = aiot_device_create(product_key, device_name);
    if (device == NULL) {
        printf("device create failed\n");
        return NULL;
    }
    /* 设置设备密钥 */
    aiot_device_set_device_secret(device, device_secret);

    /* 连接配置参数初始化 */
    aiot_linkconfig_t* config = aiot_linkconfig_init(protocol);
    /* 设置服务器的host、port */
    aiot_linkconfig_host(config, host, port);

    /* 设置设备连接参数 */
    aiot_device_set_linkconfig(device, config);
    
    /* 设备建连 */
    res = aiot_device_connect(device);
    if (res < STATE_SUCCESS) {
        /* 尝试建立连接失败, 销毁MQTT实例, 回收资源 */
        aiot_linkconfig_deinit(&config);
        aiot_device_delete(&device);
        printf("aiot_device_connect failed: -0x%04X\n\r\n", -res);
        return NULL;
    }

    /* 建连成功返回设备对象 */
    aiot_linkconfig_deinit(&config);
    return device;
}

static void demo_device_deinit(void *device)
{
    int32_t res = STATE_SUCCESS;
    res = aiot_device_disconnect(device);
    if (res < STATE_SUCCESS) {
        printf("aiot_device_disconnect failed: -0x%04X\n", -res);
    }
    aiot_device_delete(&device);
}

/* 设备标签消息回复后的回调函数 */
static void demo_devinfo_callback(void *device, const aiot_devinfo_msg_reply_t *devinfo_msg, void *userdata)
{
    if(device == NULL || devinfo_msg == NULL) {
        return;
    }

    printf("demo_devinfo_callback msg id %d\r\n", devinfo_msg->msg_id);
}

int main(int argc, char *argv[])
{
    void *device = NULL;
    /* 配置SDK的底层依赖 */
    aiot_sysdep_init(aiot_sysdep_get_portfile());

    /* 初始化设备，并将设备连接上云 */
    device = demo_device_init(product_key, device_name, device_secret, host, port);
    if(device == NULL) {
        printf("demo device connect failed\r\n");
    }

    aiot_device_devinfo_set_callback(device, demo_devinfo_callback, NULL);

    /* 测试添加标签key,value */
    aiot_device_devinfo_add(device, "test_key", "test_value");

    /* 测试删除标签key */
    /*
    aiot_device_devinfo_delete(device, "test_key");
    */
    int i = 5;
    while(i--) {
        sleep(10);
    }

    /* 断开设备连接，并回收设备资源 */
    demo_device_deinit(device);
    return 0;
}
